//////////////////////////////////////////////////////////////////////
// XLogical - A puzzle game
//
// Copyright (C) 2000 Neil Brown, Tom Warkentin
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// or at the website: http://www.gnu.org
//
////////////////////////////////////////////////////////////////////////

extern char datapath[2048];

#include <dirent.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>

#include "exception.h"
#include "modlist.h"
#include "properties.h"

bool gIsInGameFile	= false;

Cmodlist* 				Cmodlist::fGlobalList	= NULL;
Cmodlist::iterator	Cmodlist::fCurrent;

Cmodlist::Cmodlist()
{
	if (fGlobalList)
	{
		ThrowEx( "modlist already initialized" );
	}
	fGlobalList = this;
}

Cmodlist::~Cmodlist()
{
	if (fGlobalList == this)
	{
		fGlobalList = NULL;
	}
	clear();
}

void
Cmodlist::init()
{
	// clear our music lists
	clear();

	// figure out if we should load a list of file names or not
	
	char tempdata[2048];
	strncpy(tempdata,datapath,2048);
	
	string inGame = string(datapath) + properties::instance().get( "music.in_game" );
	struct stat info;
	if ((stat( inGame.c_str(), &info ) == 0) && S_ISREG(info.st_mode))
	{
		// user supplied the name of a single file
		gIsInGameFile = true;
		return;
	}

	// parse mod paths out of input parameter
	path_list_t l;
	parse_paths( inGame, l );

	// loop through finding mods in the mod paths
	DIR *dirStream = NULL;
	string filePath;
	struct dirent * dirEntry = NULL;
	struct stat s;
	for( path_list_t::iterator iter=l.begin(); iter != l.end(); ++iter )
	{
		dirStream = opendir( (*iter).c_str() );
		if (!dirStream)
		{
			ThrowEx(
				string( "unable to open dir '" ) +
				(*iter) +
				string( "'" ) );
		}

		while( (dirEntry = readdir( dirStream ) ) )
		{
			// skip current and parent dir entries
			if ((strcmp( dirEntry->d_name, "." )!=0) && 
				(strcmp( dirEntry->d_name, "..")!=0))
			{

				// build full path to music file
				filePath = (*iter);
				filePath += "/";
				filePath += dirEntry->d_name;

				// make sure file is a regular file
				if ((stat( filePath.c_str(), &s ) == 0) &&
					(S_ISREG(s.st_mode)))
				{
#if DEBUG & 2
					cout
						<< "adding mod ["
						<< filePath << "] to user list" << endl;
#endif
					push_back( filePath );
				}
			}
		}
		closedir( dirStream );
	}
	fCurrent = begin();
}

void
Cmodlist::randomize()
{
	if (size() > 1)
	{
		file_list_base_t tmpList;

		struct timeval tv;
		gettimeofday( &tv, NULL );

		srandom( tv.tv_sec + tv.tv_usec );
		tmpList.swap( *this );
		for( file_list_base_t::iterator iter=tmpList.begin();
			  iter!=tmpList.end();
			  iter=tmpList.begin() )
		{
			string s;
			int n = random() % tmpList.size();

			for(int i=0;i<n;i+=1)
			{
				++iter;
			}
			s = *iter;
			tmpList.remove( *iter );
			push_back( s );
		}
	}
}

Cmodlist&
Cmodlist::instance()
{
	if (!fGlobalList)
	{
		ThrowEx( "modlist not initialized" );
	}
	return( *fGlobalList );
}

void
Cmodlist::parse_paths( const string& aDirPaths, path_list_t& aOutputPaths )
{
	bool done = false;
	string paths( aDirPaths );
	string::size_type i;

	aOutputPaths.clear();
	while( !done )
	{
		i = paths.find( kPathSeparator );
		if ((i != string::npos) && (i > 0))
		{
			string s( paths, 0, i );
			aOutputPaths.push_back( s );
			paths.erase( 0, i+1 );
		} else {
			if (paths.size() > 0)
			{
				aOutputPaths.push_back( paths );
			}
			done = true;
		}
	}
}

string
Cmodlist::next()
{
	if (gIsInGameFile)
	{
		return( properties::instance().get( "music.in_game" ) );
	}
	if (empty())
	{
		return( string( "" ) );
	}
	if (size() > 1)
	{
		++fCurrent;
		if (fCurrent == end())
		{
			fCurrent = begin();
		}
	} else {
		fCurrent = begin();
	}
	return( *fCurrent );
}

string
Cmodlist::previous()
{
	if (gIsInGameFile)
	{
		return( properties::instance().get( "music.in_game" ) );
	}
	if (empty())
	{
		return( string("") );
	}
	if (size() > 1)
	{
		if (fCurrent == begin())
		{
			fCurrent = end();
			--fCurrent;
		} else {
			--fCurrent;
		}
	} else {
		fCurrent = begin();
	}
	return( *fCurrent );
}
