//////////////////////////////////////////////////////////////////////
// XLogical - A puzzle game
//
// Copyright (C) 2000 Neil Brown, Tom Warkentin
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// or at the website: http://www.gnu.org
//
////////////////////////////////////////////////////////////////////////




// Application Includes
#include "audio.h"
#include "properties.h"
#include "tiles.h"
#include "graph.h"
#include "gamelogic.h"

//#define DEBUG_FUNC

Ctile_oneway::Ctile_oneway( )
{
#ifdef DEBUG_FUNC
	cerr << __FILE__ << " Ctile_oneway::constructor( )" << endl;
#endif
	exitFlags = 0l;
	oneWayDir = UP;
}

Ctile_oneway::Ctile_oneway( unsigned long flags, dir_t oneWay )
{
#ifdef DEBUG_FUNC
	cerr << __FILE__ << " Ctile_oneway::constructor( long )" << endl;
#endif
	exitFlags = flags;
	oneWayDir = oneWay;
}

Ctile_oneway::~Ctile_oneway( )
{
#ifdef DEBUG_FUNC
	cerr << __FILE__ << " Ctile_oneway::destructor( )" << endl;
#endif
}

void
Ctile_oneway::move_balls( void )
{
#ifdef DEBUG_FUNC1
	cerr << __FILE__ << " Ctile_oneway::move_balls( )" << endl;
#endif

	float	xcoord		= 0;
	float	ycoord		= 0;
	float	overShoot	= 0;
	dir_t	dir			= UP;
	int		halfTile	= CURMAP->tileSize / 2;
	int 	overHalf	= 0;
	list< Cball * >::iterator it;

	// Bail if there are no balls here
	if( balls.empty( ) )
	{
		return;
	}

	it = balls.begin( );

	while( it != balls.end( ) )
	{
		// Get the direction the ball is moving in
		dir = (*it)->get_direction( );

		// Get the ball's coordinates
		(*it)->get_coordinates( &xcoord, &ycoord);

		// Clear the no move flag if it's set
		if( (*it)->check_no_move( ) )
		{
			(*it)->clear_no_move( );
			it++;
		} else {

			// Reset our flag
			overHalf = 0;

			// Find out if the ball is going to cross the mid point of
			// the tile.
			switch( dir )
			{
			case UP:
				if( (ycoord > halfTile )
					&& ( ycoord - CURMAP->ballSpd <= halfTile ) )
				{
					overShoot = fabs( ycoord - CURMAP->ballSpd - halfTile );
					overHalf = 1;
				}
				break;
			case DOWN:
				if( (ycoord < halfTile )
					&& ( ycoord + CURMAP->ballSpd >= halfTile ) )
				{
					overShoot = fabs( ycoord + CURMAP->ballSpd - halfTile );
					overHalf = 1;
				}
				break;
			case LEFT:
				if( (xcoord > halfTile )
					&& ( xcoord - CURMAP->ballSpd <= halfTile ) )
				{
					overShoot = fabs( ycoord - CURMAP->ballSpd - halfTile );
					overHalf = 1;
				}
				break;
			case RIGHT:
				if( (xcoord < halfTile )
					&& ( xcoord + CURMAP->ballSpd >= halfTile ) )
				{
					overShoot = fabs( xcoord + CURMAP->ballSpd - halfTile );
					overHalf = 1;
				}
				break;
			default:
				break;
			}

			// Will we go over the halfway point after this move?
			if( overHalf )
			{
				// Move the ball manually since we know what to do
				switch( oneWayDir )
				{
				case UP:
					//exitFlags = EXIT_UP;
					xcoord = halfTile;
					ycoord = halfTile - overShoot;
					break;
				case DOWN:
					//exitFlags = EXIT_DOWN;
					xcoord = halfTile;
					ycoord = halfTile + overShoot;
					break;
				case LEFT:
					//exitFlags = EXIT_LEFT;
					xcoord = halfTile - overShoot;
					ycoord = halfTile;
					break;
				case RIGHT:
					//exitFlags = EXIT_RIGHT;
					xcoord = halfTile + overShoot;
					ycoord = halfTile;
					break;
				default:
					break;
				}
				audioDriver->play_sound(
					properties::instance().get( "sound.one_way" ).c_str() );
				(*it)->set_coordinates( xcoord, ycoord );
				(*it)->set_direction( oneWayDir );
				it++;

			} else {

				// Move the ball with the base class functions
				if( move_a_ball( **it ) == 1 )
				{
					// It left the tile, remove it
					it = balls.erase( it );
				} else {
					// Move on
					it++;
				}
			}
		}
	}
}

void
Ctile_oneway::draw_over( int ulX, int ulY )
{
#ifdef DEBUG_FUNC1
	cerr << __FILE__ << " Ctile_oneway::draw_over( ";
	cerr << ulX << ", " << ulY << " )" << endl;
#endif
	int pm = -1;
	
	// Draw the base for the oneway
	graphDriver->graph_draw( BMP_NEXT_BASE, ulX, ulY );

	// Now draw the oneway
	switch( oneWayDir )
	{
	case UP:
		pm = BMP_ARROW_U;
		break;
	case DOWN:
		pm = BMP_ARROW_D;
		break;
	case LEFT:
		pm = BMP_ARROW_L;
		break;
	case RIGHT:
		pm = BMP_ARROW_R;
		break;
	default:
		break;
	}

	graphDriver->graph_draw( pm, ulX + 23, ulY + 23 );
}
// $Id: tile_oneway.cc,v 1.5 2000/10/08 02:53:49 brown Exp $
//
// $Log: tile_oneway.cc,v $
// Revision 1.5  2000/10/08 02:53:49  brown
// Fixed the level editor and some graphics
//
// Revision 1.4  2000/10/07 18:16:34  brown
// Ball movement fixes
//
// Revision 1.3  2000/10/06 19:29:12  brown
// Added autoconf stuff
// Added GPL header to files
// Added WON scroller
// Added GPL to about screen
//
// Revision 1.2  2000/10/01 05:00:26  tom
// put in infrastructure for abstract audio drivers and started to code
// SDL_mixer support.
//
// Revision 1.1.1.1  2000/09/28 02:17:52  tom
// imported sources
//
// Revision 1.4  2000/03/28 07:31:10  tom
// Added calls to play more sound effects.
//
// Revision 1.3  1999/12/25 08:18:40  tom
// Added "Id" and "Log" CVS keywords to source code.
//
